package com.divillysausages.escvel.gui 
{
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Trade;
	import com.divillysausages.escvel.logic.Planet;
	import com.divillysausages.escvel.logic.PlayerShip;
	import com.greensock.easing.Elastic;
	import com.greensock.TweenLite;
	import flash.accessibility.AccessibilityProperties;
	import flash.display.Sprite;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class BuyDialog extends Sprite
	{
		private var m_planetName:TextField 	= null;
		private var m_excessTxt:TextField	= null;
		private var m_sellTxt:TextField		= null;		
		private var m_resource1:TradeButton	= null;
		private var m_resource2:TradeButton	= null;
		private var m_sellButton:SellButton	= null;
		private var m_done:DoneButton		= null;
		private var m_player:PlayerShip		= null;
		
		public function set planet( p:Planet ):void
		{
			// set the planet name
			this.m_planetName.text 	= p.name;
			this.m_planetName.x		= this.width * 0.5 - this.m_planetName.width * 0.5;
			
			// set the resources
			this.m_resource1.setTrade( p.port.excessGood1, Trade.getBasePrice( p.port.excessGood1 ) );
			this.m_resource2.setTrade( p.port.excessGood2, Trade.getBasePrice( p.port.excessGood2 ) );
		}
		
		public function set player( p:PlayerShip ):void
		{
			this.m_player = p;
			
			// the buy buttons
			// the player is full or can't afford either resource
			if ( this.m_player.cargoAmount == this.m_player.maxCargoAmount || ( this.m_player.credits < this.m_resource1.price && this.m_player.credits < this.m_resource2.price ) )
			{
				this.m_resource1.enabled = false;
				this.m_resource2.enabled = false;
			}
			else
			{
				// enabled/disable the button
				this.m_resource1.enabled = ( this.m_player.credits >= this.m_resource1.price );
				this.m_resource2.enabled = ( this.m_player.credits >= this.m_resource2.price );
			}
			
			// the sell button
			if ( this.m_player.cargo == null )
				this.m_sellButton.enabled = false;
			else
			{
				this.m_sellButton.setTrade( this.m_player.cargo, Trade.getBasePrice( this.m_player.cargo ) );
				this.m_sellButton.enabled = true;
			}
		}
		
		public function BuyDialog() 
		{
			// create our name textfield
			var tf:TextFormat 	= new TextFormat( Resources.GAME_FONT, 20.0 );
			this.m_planetName	= this._createTextField( "Planet name", tf );
			this.m_planetName.y	= 10.0;
			this.addChild( this.m_planetName );
			
			// create the planet selling area
			tf					= new TextFormat( Resources.GAME_FONT, 10.0 );
			this.m_excessTxt 	= this._createTextField( "Buy some resources:", tf );
			this.m_excessTxt.x	= 10.0;
			this.m_excessTxt.y	= this.m_planetName.y + this.m_planetName.height + 20.0;
			this.addChild( this.m_excessTxt );
			
			// create the resource buttons
			this.m_resource1 	= new TradeButton;
			this.m_resource1.x	= 10.0;
			this.m_resource1.y	= this.m_excessTxt.y + this.m_excessTxt.height + 5.0;
			this.m_resource1.onClick.add( this._onBuy );
			this.addChild( this.m_resource1 );
			
			this.m_resource2	= new TradeButton;
			this.m_resource2.x	= this.m_resource1.x + this.m_resource1.width + 10.0;
			this.m_resource2.y	= this.m_resource1.y;
			this.m_resource2.onClick.add( this._onBuy );
			this.addChild( this.m_resource2 );
			
			// create the sell text
			this.m_sellTxt		= this._createTextField( "Sell:", tf );
			this.m_sellTxt.x	= 10.0;
			this.m_sellTxt.y	= this.m_resource1.y + this.m_resource1.height + 20.0;
			this.addChild( this.m_sellTxt );
			
			// create the sell button
			this.m_sellButton 	= new SellButton;
			this.m_sellButton.x	= 10.0;
			this.m_sellButton.y	= this.m_sellTxt.y + this.m_sellTxt.height + 5.0;
			this.m_sellButton.onClick.add( this._onSell );
			this.addChild( this.m_sellButton );
			
			// get our width
			var w:Number = this.m_resource1.width + this.m_resource2.width + 30.0; // padding all around
			
			// create our done button
			this.m_done 		= new DoneButton;
			this.m_done.x		= w * 0.5 - this.m_done.width * 0.5;
			this.m_done.y		= this.m_sellButton.y + this.m_sellButton.height + 20.0;
			this.m_done.onClick.add( this._onClose );
			this.addChild( this.m_done );
			
			// get our height
			var h:Number = this.m_done.y + this.m_done.height + 10.0;
			
			// draw our bg
			this.graphics.lineStyle( 2.0, 0x666666 );
			this.graphics.beginFill( 0xdddddd );
			this.graphics.drawRoundRect( 0.0, 0.0, w, h, 15.0, 15.0 );
			this.graphics.endFill();
		}
		
		private function _createTextField( label:String, tf:TextFormat ):TextField
		{
			var t:TextField 	= new TextField;
			t.autoSize			= TextFieldAutoSize.LEFT;
			t.defaultTextFormat	= tf;
			t.text				= label;
			t.selectable		= false;
			t.mouseEnabled		= false;
			t.mouseWheelEnabled	= false;
			t.embedFonts		= true;
			return t;
		}
		
		private function _onClose( d:DoneButton ):void
		{
			TweenLite.to( this, 1.0, { y: -this.height * 1.5, ease: Elastic.easeOut } );
		}
		
		private function _onSell( s:SellButton ):void
		{
			// tell the player to sell
			this.m_player.sell( s.price );
			
			// clear the button
			s.setTrade( null, 0 );
			s.enabled = false;
			
			// enable the other buttons
			this.m_resource1.enabled = ( this.m_player.credits >= this.m_resource1.price );
			this.m_resource2.enabled = ( this.m_player.credits >= this.m_resource2.price );
		}
		
		private function _onBuy( t:TradeButton ):void
		{
			// tell the player to buy
			this.m_player.buy( null, t.trade, t.price );
			
			// close the panel
			this._onClose( null );
		}
		
	}

}