package com.divillysausages.escvel.gui 
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.escvel.assets.Resources;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.filters.GlowFilter;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Instructions extends Sprite
	{
		private static const TF:TextFormat = new TextFormat(Resources.GAME_FONT, 8.0, 0x666666);
		
		private var m_keys:TextField		= null;
		private var m_mouse:Bitmap			= null;
		private var m_credits:TextField		= null;
		private var m_paused:TextField		= null;
		private var m_showing:Boolean 		= false;
		private var m_currShowTime:Number	= 0.0;
		private var m_showTime:Number		= 1.0;
		
		public function Instructions() 
		{
			// mouse
			this.m_mouse			= new Bitmap( DS.getAsset( Resources.MOUSE_IMAGE ).bitmapData );
			this.m_mouse.scaleX		= 2.0;
			this.m_mouse.scaleY		= 2.0;
			this.m_mouse.x			= DS.stage.stageWidth * 0.5 - this.m_mouse.width * 0.5;
			this.m_mouse.y			= DS.stage.stageHeight * 0.5 + 50.0;
			this.m_mouse.alpha		= 0.0;
			this.m_mouse.filters 	= [new GlowFilter( 0xffffbe, 1.0, 6, 6 )];
			this.addChild( this.m_mouse );
			
			// keys
			this.m_keys 	= this._createTextField( "[R] Reset - [S] SFX - [M] Music - [D] Drop cargo" );
			this.m_keys.x	= 10.0;
			this.m_keys.y	= 10.0;
			
			// credits
			this.m_credits 		= this._createTextField( "Tiny Trader - Ludum Dare #23" );
			this.m_credits.x 	= DS.stage.stageWidth - this.m_credits.width - 10.0;
			this.m_credits.y	= 10.0;
			
			// paused
			this.m_paused 	= this._createTextField( "Game paused - Press [P] to unpause" );
			this.m_paused.x = DS.stage.stageWidth * 0.5 - this.m_paused.width * 0.5;
			this.m_paused.y = DS.stage.stageHeight * 0.5 - this.m_paused.height * 0.5;
			this.removeChild( this.m_paused );
			
			// add to stage
			DS.stage.addChild( this );
			DS.onPauseChange = this._onPauseChange;
			
			this.mouseChildren 	= false;
			this.mouseEnabled 	= false;
		}
		
		public function start():void
		{
			this.m_showing 		= true;
			this.m_mouse.alpha	= 0.0;
			this.m_currShowTime	= 0.0;
		}
		
		public function update( dt:Number ):void
		{
			if ( !this.m_showing && this.m_mouse.alpha <= 0.0 )
				return;
				
			// showing
			if ( this.m_showing )
			{
				if ( Input.isMousePressed )
					this.m_showing = false;
					
				// fade in
				if ( this.m_mouse.alpha < 1.0 )
					this.m_mouse.alpha += 1.0 * dt;
				else
				{
					// stay for a bit
					this.m_mouse.alpha 	= 1.0;
					this.m_currShowTime += dt;
					if ( this.m_currShowTime > this.m_showTime )
						this.m_showing = false;
				}
			}
			else
				this.m_mouse.alpha -= 1.0 * dt; // fade out
		}
		
		private function _createTextField( label:String ):TextField
		{
			var t:TextField 	= new TextField;
			t.autoSize 			= TextFieldAutoSize.LEFT;
			t.defaultTextFormat	= Instructions.TF;
			t.selectable		= false;
			t.mouseEnabled		= false;
			t.mouseWheelEnabled	= false;
			t.embedFonts		= true;
			t.text				= label;
			this.addChild( t );
			
			return t;
		}
		
		private function _onPauseChange( isPaused:Boolean ):void
		{
			if ( isPaused )
				this.addChild( this.m_paused );
			else if ( this.m_paused.parent != null )
				this.m_paused.parent.removeChild( this.m_paused );
		}
		
	}

}