package com.divillysausages.escvel.gui  
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.particles.Emitter;
	import com.divillysausages.ds.render.BlitCanvas;
	
	/**
	 * The ParticleRenderer handles the update and drawing of the particles in the game
	 * @author Damian
	 */
	public class ParticleRenderer extends BlitCanvas implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		private var m_active:Boolean			= true;// are we currently active?
		private var m_emitters:Vector.<Emitter> = null;	// our list of emitters
		
		/*******************************************************************************************/
		
		/**
		 * Are we currently active?
		 */
		public function get active():Boolean { return this.m_active; }
		public function set active( b:Boolean ):void
		{
			this.m_active = b;
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates the Particle renderer
		 */
		public function ParticleRenderer() 
		{
			super( 300.0, 300.0 );
			
			this.m_emitters = new Vector.<Emitter>;
			
			// add us to the update
			DS.addToUpdate( this );
		}
		
		/**
		 * Adds a particle emitter to be drawn by this
		 * @param e	The {@link Emitter} to add
		 */
		public function addEmitter( e:Emitter ):void
		{
			var index:int = this.m_emitters.indexOf( e );
			if ( index != -1 )
				return;
				
			// add the emitter
			this.m_emitters.push( e );
			e.addToRenderList( this );
		}
		
		/**
		 * Removes a particle emitter from this canvas
		 * @param e	The {@link Emitter} to remove
		 */
		public function removeEmitter( e:Emitter ):void
		{
			var index:int = this.m_emitters.indexOf( e );
			if ( index == -1 )
				return;
				
			// remove the emitter
			this.m_emitters.splice( index, 1 );				
			e.removeFromRenderList( this );
		}
		
		/**
		 * Updates the particle renderer and draws if they're any active particles
		 * @param dt	The time since the last call to update
		 */
		public function update( dt:Number ):void
		{
			// if any are active, then draw
			var draw:Boolean	= false;
			var len:int 		= this.m_emitters.length;
			for ( var i:int = 0; i < len; i++ )
			{
				if ( this.m_emitters[i].active )
				{
					draw = true;
					break;
				}
			}
			
			// draw only if we have to
			if ( draw )
				this.draw();
				
			this.visible = draw;
		}
		
	}

}