package com.divillysausages.escvel.gui 
{
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Mood;
	import com.divillysausages.escvel.constants.Trade;
	import com.divillysausages.escvel.logic.ObjectPools;
	import com.divillysausages.escvel.logic.PlayerShip;
	import flash.display.Sprite;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class PlayerHUD extends Sprite
	{
		private static const TF:TextFormat = new TextFormat( Resources.GAME_FONT, 12.0, 0xffffff );
		
		public var cargoTxt:TextField 	= null;
		public var moneyTxt:TextField	= null;
		
		private var m_prevCredits:int = 0;
		
		public function set player( p:PlayerShip ):void
		{
			// add our event listners
			p.signalCredits.add( this._onCreditsChange );
			this._onCreditsChange( p.credits );
			
			p.signalCargo.add( this._onCargoChange );
			this._onCargoChange( p.cargo, p.cargoAmount, p.maxCargoAmount );
		}
		
		public function PlayerHUD() 
		{
			// money display
			this.moneyTxt 	= this._createTextField( "Money: 0 credits" );
			this.addChild( this.moneyTxt );
			
			// cargo text
			this.cargoTxt 	= this._createTextField( "Cargo: none" );
			this.cargoTxt.x	= 180.0;
			this.addChild( this.cargoTxt );
			
			this.mouseEnabled 	= false;
			this.mouseChildren	= false;
		}
		
		private function _createTextField( label:String ):TextField
		{
			var t:TextField 	= new TextField;
			t.autoSize			= TextFieldAutoSize.LEFT;
			t.defaultTextFormat	= PlayerHUD.TF;
			t.text				= label;
			t.selectable		= false;
			t.mouseEnabled		= false;
			t.mouseWheelEnabled	= false;
			t.embedFonts		= true;
			return t;
		}
		
		private function _onCreditsChange( amount:int ):void
		{
			this.moneyTxt.text = "Money: " + amount + " credits";
			if ( this.m_prevCredits != 0 )
			{
				var diff:int 	= amount - this.m_prevCredits;
				var msg:String	= ( ( diff > 0 ) ? "+" + diff : diff ) + " credits";
				var mood:Mood	= ( diff > 0 ) ? Mood.GOOD : Mood.BAD;
				ObjectPools.getFeedback( msg, mood, this.x + this.moneyTxt.x, this.y + this.moneyTxt.y );
			}
			
			this.m_prevCredits = amount;
		}
		
		private function _onCargoChange( trade:Trade, amount:int, max:int ):void
		{
			if ( trade == null )
				this.cargoTxt.text = "Cargo: none: " + amount + "/" + max;
			else
			{
				this.cargoTxt.text = "Cargo: " + Trade.toString( trade ) + ": " + amount + "/" + max + " (" + ( amount * Trade.getBasePrice( trade ) ) + "c)";
				
				var msg:String	= "+" + amount + " " + Trade.toString( trade );
				var mood:Mood	= Mood.GOOD;
				ObjectPools.getFeedback( msg, mood, this.x + this.cargoTxt.x, this.y + this.cargoTxt.y );
			}
		}
		
	}

}