package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.collections.objectpool.IPoolObj;
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import flash.display.Shape;
	import flash.geom.Point;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class DirArrow extends Shape implements IUpdateObj, IPoolObj
	{
		public var dir:Point 			= new Point;
		public var player:PlayerShip	= null;
		public var dist:Number			= 0.0;
		public var planet:Planet		= null;
		
		public function get active():Boolean { return this.visible; }
		
		public function DirArrow() 
		{
			this.draw();
			DS.addToUpdate( this );
			this.visible = false;
		}
		
		public function init(params:* = null):void 
		{
			this.planet = params as Planet;
			this.planet.parent.addChild( this );
			
			// show it so it updates
			this.visible = true;
		}
		
		public function destroy():void 
		{
			DS.removeFromUpdate( this );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.player	= null;
			this.planet	= null;
			this.dir	= null;
		}
		
		public function reset():void 
		{
			this.visible = false;
		}
		
		public function update( dt:Number ):void
		{
			if ( this.player == null )
				return;
			
			// rotate towards the planet
			this.dir.x 	= ( this.planet.x - this.player.x );
			this.dir.y 	= ( this.planet.y - this.player.y );
			this.dist	= this.dir.length;
			this.dir.normalize( 1.0 );
			this.rotation = MathHelper.radiansToDegrees( MathHelper.radianAngle( this.dir ) );
			
			// set the alpha - further away, lighter
			var alphaDist:Number = MathHelper.clamp( this.dist, 100.0, 1000.0 );
			this.alpha = 0.2 + 0.8 * ( 1.0 - ( alphaDist / 1000.0 ) );
		}
		
		public function draw():void
		{
			this.graphics.clear();
			this.graphics.lineStyle( 2.0, 0xffffff );
			this.graphics.moveTo( -2.5, -5.0 );
			this.graphics.lineTo( 2.5, 0.0 );
			this.graphics.lineTo( -2.5, 5.0 );
		}
		
	}

}