package com.divillysausages.escvel.logic 
{
	import com.divillysausages.escvel.constants.PlanetNames;
	import flash.display.Shape;
	import flash.display.Sprite;
	/**
	 * A galaxy holds all the planets
	 * @author Damian Connolly
	 */
	public class Galaxy extends Sprite
	{
		
		public var planets:Vector.<Planet> 	= null;
		public var bounds:Number			= 2000.0;
		
		private var m_closeDist:Number	= 10000.0;
		private var m_tradeDist:Number	= 80000.0;
		private var m_tradeRoutes:Shape	= null;
		
		public function Galaxy() 
		{
			// create about 50 
			var len:int = PlanetNames.names.length;
			this.planets = new Vector.<Planet>( len, true );
			for ( var i:int = 0; i < len; i++ )
			{
				var p:Planet 	= new Planet;
				p.name			= PlanetNames.names[i];
				this.addChild( p );
				this.planets[i] = p;
				
				// position
				p.position( this.bounds );
				
				// make sure they're not too close to the rest
				for ( var j:int = 0; j < len; j++ )
				{
					var p2:Planet = this.planets[j];
					if ( p == p2 || p2 == null )
						continue;
						
					// if it's too close, reposition it
					if ( p.dist( p2 ) < this.m_closeDist )
					{
						p.position( this.bounds );
						j = 0; // reset to check again
					}
				}
			}
			
			// for drawing our trade routes
			this.m_tradeRoutes = new Shape;
			this.m_tradeRoutes.graphics.lineStyle( 1.0, 0xffffff, 0.1 );
			
			// go through and connect them up for trade
			for ( i = 0; i < len; i++ )
			{
				p = this.planets[i];
				for ( j = 0; j < len; j++ )
				{
					p2 = this.planets[j];
					if ( p == p2 )
						continue; // ignore, same planet
						
					// if it's close enough for trade, add it
					if ( p.dist( p2 ) <= this.m_tradeDist )
					{
						p.addTrade( p2 );
						p2.addTrade( p );
						
						// draw the route
						this.m_tradeRoutes.graphics.moveTo( p.x, p.y );
						this.m_tradeRoutes.graphics.lineTo( p2.x, p2.y );
					}
				}
			}
			
			// create the trade ships
			for ( i = 0; i < len; i++ )
				this.planets[i].createTradeShips();
			
			// add it under all the planets
			this.addChildAt( this.m_tradeRoutes, 0 );
			
			this.mouseEnabled 	= false;
			this.mouseChildren 	= false;
		}
		
		public function getPlanetAtPos( x:Number, y:Number ):Planet
		{
			// go through all our planets and check them
			for each( var p:Planet in this.planets )
			{
				var dx:Number = p.x - x;
				var dy:Number = p.y - y;
				var dist:Number = ( dx * dx + dy * dy );
				if ( dist <= p.radius * p.radius )
					return p;
			}
			return null;
		}
		
	}

}