package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.math.MathHelper;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.filters.GlowFilter;
	import flash.geom.Matrix;
	import flash.utils.Dictionary;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class MiniMap extends Sprite
	{
		private var m_galaxy:Bitmap 		= new Bitmap;
		private var m_player:Shape			= new Shape;
		private var m_pirates:Dictionary	= new Dictionary;
		private var m_bounds:Number			= 0.0;
		
		public function MiniMap( galaxy:Galaxy ) 
		{
			this.m_galaxy.bitmapData = new BitmapData( 100, 100, false, 0 );
			
			// draw our galaxy
			var m:Matrix = new Matrix;
			m.translate( galaxy.bounds * 0.5, galaxy.bounds * 0.5 );
			m.scale( this.m_galaxy.bitmapData.width / galaxy.bounds, this.m_galaxy.bitmapData.height / galaxy.bounds );
			this.m_galaxy.bitmapData.draw( galaxy, m );
			this.addChild( this.m_galaxy );
			
			// draw our shape
			this.m_player.graphics.beginFill( 0x009900 );
			this.m_player.graphics.drawCircle( 0.0, 0.0, 3.0 );
			this.m_player.graphics.endFill();
			
			// position and add it
			this.m_player.x = this.width * 0.5;
			this.m_player.y = this.height * 0.5;
			this.addChild( this.m_player );
			
			// hold our bounds
			this.m_bounds = galaxy.bounds;
			
			this.filters = [new GlowFilter(0xffffbe)];
			
			this.mouseEnabled	= false;
			this.mouseChildren 	= false;
		}
		
		public function update( player:PlayerShip ):void
		{
			var dx:Number = player.x / this.m_bounds;
			var dy:Number = player.y / this.m_bounds;
			dx = MathHelper.clamp( dx, -0.5, 0.5 );
			dy = MathHelper.clamp( dy, -0.5, 0.5 );
			
			// update the player ship position
			this.m_player.x = this.m_galaxy.width * 0.5 + this.m_galaxy.width * dx;
			this.m_player.y = this.m_galaxy.height * 0.5 + this.m_galaxy.height * dy;
		}
		
		public function updatePirate( pirate:Pirate ):void
		{
			var s:Shape = this.m_pirates[pirate];
			if ( s == null )
			{
				s = new Shape;
				s.graphics.beginFill( 0xff9999 );
				s.graphics.drawCircle( 0.0, 0.0, 3.0 );
				s.graphics.endFill();
				this.addChild( s );
				this.m_pirates[pirate] = s;
			}
			
			// get the position
			var dx:Number = pirate.x / this.m_bounds;
			var dy:Number = pirate.y / this.m_bounds;
			dx = MathHelper.clamp( dx, -0.5, 0.5 );
			dy = MathHelper.clamp( dy, -0.5, 0.5 );
			
			// update the player ship position
			s.x = this.m_galaxy.width * 0.5 + this.m_galaxy.width * dx;
			s.y = this.m_galaxy.height * 0.5 + this.m_galaxy.height * dy;
		}
		
	}

}