package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.escvel.assets.Resources;
	import flash.display.Bitmap;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.Point;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Pirate extends Sprite implements IUpdateObj
	{
		public var speed:Number 		= 20.0;
		public var maxSpeed:Number		= 1.2;
		public var shieldRadius:Number	= 25.0;
		public var shield:Sprite		= null;
		
		private var m_dir:Point 			= new Point;
		private var m_galaxy:Galaxy			= null;
		private var m_velX:Number			= 0.0;
		private var m_velY:Number			= 0.0;
		private var m_player:PlayerShip		= null;
		private var m_target:Point			= null;
		private var m_currDirTime:Number	= 0.0;
		private var m_dirTime:Number		= 2.0;
		private var m_currHitTime:Number	= 0.0;
		private var m_hitTime:Number		= 0.5;
		
		// for colliding with the player
		private var m_minMoveDist:Point = new Point;
		private var m_velDiff:Point		= new Point;
		
		public function get active():Boolean { return true; }
		
		public function Pirate( galaxy:Galaxy, player:PlayerShip ) 
		{
			this.m_galaxy	= galaxy;
			this.m_player	= player;
			this.draw();
			
			// add shield
			this.shield 		= new Sprite;
			this.shield.alpha	= 0.0;
			var b:Bitmap		= new Bitmap( DS.getAsset( Resources.PIRATE_SHIELD_IMAGE ).bitmapData );
			b.x					= -b.width * 0.5;
			b.y					= -b.height * 0.5;
			this.shield.addChild( b );
			
			DS.addToUpdate( this );
		}
		
		public function changeDir():void
		{
			this.m_target.x += -100.0 + Math.random() * 200.0;
			this.m_target.y += -100.0 + Math.random() * 200.0;
			
			// clamp them so it doesn't go outside the bounds
			var bw:Number = this.m_galaxy.bounds * 0.5 + 100.0;
			this.m_target.x = MathHelper.clamp( this.m_target.x, -bw, bw );
			this.m_target.y = MathHelper.clamp( this.m_target.y, -bw, bw );
			
			// keep them out of the center
			//bw -= 200.0;
			//if ( this.m_target.x > -bw && this.m_target.x < bw && this.m_target.y > -bw && this.m_target.y < bw )
			//{
				//var p:Point = this.m_target.clone();
				//p.normalize( 1.0 );
				//this.m_target.x += p.x * 20.0;
				//this.m_target.y += p.y * 20.0;
			//}
				
			this.m_currDirTime = 0.0; // reset dir time
		}
		
		public function update( dt:Number ):void
		{				
			// get our target
			if ( this.m_target == null )
			{
				this.m_target 	= new Point;
				this.m_target.x	= this.x;
				this.m_target.y	= this.y;
				this.changeDir();
			}
			
			// get the direction to the player
			var dx:Number 	= this.m_player.x - this.x;
			var dy:Number 	= this.m_player.y - this.y;
			var dist:Number	= Math.sqrt( dx * dx + dy * dy );
			if ( dist < 200.0 )
			{
				// we're targeting the player
				this.m_dir.x = dx;
				this.m_dir.y = dy;
			}
			else
			{
				this.m_currDirTime += dt;
				if ( this.m_currDirTime > this.m_dirTime )
					this.changeDir();
					
				// we're just doing our normal shit
				this.m_dir.x = this.m_target.x - this.x;
				this.m_dir.y = this.m_target.y - this.y;
				
				// if too close, change
				if ( this.m_dir.length < 10.0 )
					this.changeDir();
			}
			
			// look at our dir
			var r:Number 	= MathHelper.radiansToDegrees( MathHelper.radianAngle( this.m_dir ) );
			this.rotation 	= MathHelper.lerp( this.rotation, r, 0.1 );
			
			// normalise dir
			this.m_dir.normalize( 1.0 );
			
			// add our vel
			this.m_velX += this.m_dir.x * this.speed * dt;
			this.m_velY += this.m_dir.y * this.speed * dt;
				
			// check the distance to the player
			this.m_currHitTime += dt;
			if ( this.m_currHitTime >= this.m_hitTime ) // so they don't continuously hit each other
				this.m_currHitTime = this.m_hitTime;
			if ( dist < this.shieldRadius + this.m_player.shieldRadius )
			{
				// only tell the player if the time is good
				if ( this.m_currHitTime >= this.m_hitTime )
				{
					this.m_player.collideWithPirate( this );
					this.m_currHitTime = 0.0;
				}
				
				// show our shield
				this.shield.rotation 	= MathHelper.radiansToDegrees( MathHelper.radianAngle( this.m_dir ) );
				this.shield.alpha 		= 1.0;
				
				// collision code taken from 
				// http://stackoverflow.com/questions/345838/ball-to-ball-collision-detection-and-handling
				
				dx 		= this.x - this.m_player.x;
				dy 		= this.y - this.m_player.y;
				dist	= Math.sqrt( dx * dx + dy * dy );
				
				// the minimum transfer distance
				var mtdx:Number = dx * ( this.shieldRadius + this.m_player.shieldRadius - dist ) / dist;
				var mtdy:Number = dy * ( this.shieldRadius + this.m_player.shieldRadius - dist ) / dist;
				dist = Math.sqrt( mtdx * mtdx + mtdy * mtdy );
				
				// inverse mass for the pirate and player
				var im1:Number = 1.0 / 3.0;
				var im2:Number = 1.0 / 1.0;
				
				// move them apart based on mass
				this.x 			+= mtdx * ( im1 / ( im1 + im2 ) );
				this.y 			+= mtdy * ( im2 / ( im1 + im2 ) );
				this.m_player.x -= mtdx * ( im2 / ( im1 + im2 ) );
				this.m_player.y -= mtdy * ( im2 / ( im1 + im2 ) );
				
				// impact speed
				this.m_velDiff.x = this.m_velX - this.m_player.vel.x;
				this.m_velDiff.y = this.m_velY - this.m_player.vel.y;
				this.m_velDiff.normalize( 1.0 );
				var vn:Number = MathHelper.dot2( this.m_velDiff.x, this.m_velDiff.y, mtdx / dist, mtdy / dist );
				
				// colliding and heading towards each other
				if ( vn < 0.0 )
				{
					// collision impulse (0.5 is restitution)
					var impluse:Number 	= ( -(1.0 + 0.5 ) * vn ) / ( im1 + im2 );
					var ix:Number 		= mtdx * impluse;
					var iy:Number 		= mtdy * impluse;
					
					// update the momentum
					this.m_velX += ix * im1;
					this.m_velY += iy * im1;
					this.m_player.addVel( -ix * im2, -iy * im2 );
				}
			}
				
			// clamp our vel
			MathHelper.clamp( this.m_velX, -this.maxSpeed, this.maxSpeed );
			MathHelper.clamp( this.m_velY, -this.maxSpeed, this.maxSpeed );
				
			// move
			this.x += this.m_velX;
			this.y += this.m_velY;
			
			// update our shield
			this.shield.x = this.x;
			this.shield.y = this.y;
			if ( this.shield.parent == null )
				this.parent.addChild( this.shield );
			if ( this.shield.alpha > 0.0 )
				this.shield.alpha -= 0.1;
			
			// reduce vel
			this.m_velX *= 0.9;
			this.m_velY *= 0.9;
			
			if ( this.m_velX <= 0.01 && this.m_velX >= -0.01 )
				this.m_velX = 0.0;
			if ( this.m_velY <= 0.01 && this.m_velY >= -0.01 )
				this.m_velY = 0.0;
		}
		
		public function draw():void
		{
			var b:Bitmap	= new Bitmap( DS.getAsset( Resources.PIRATE_IMAGE ).bitmapData );
			b.y				= -b.height * 0.5;
			b.x				= -5;
			this.addChild( b );
			//this.graphics.clear();
			//this.graphics.lineStyle( 1.0, 0x750e0e );
			//this.graphics.beginFill( 0xcf3232 );
			//
			//var h:Number = 5;
			//
			//this.graphics.moveTo( -h, -h );
			//this.graphics.lineTo( h * 2.0, 0.0 );
			//this.graphics.lineTo( -h, h );
			//this.graphics.lineTo( -h, -h );
			//
			//this.graphics.endFill();
		}
		
	}

}