package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.util.Colour;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Trade;
	import com.divillysausages.escvel.gui.PickUp;
	import flash.display.Bitmap;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.filters.GlowFilter;
	
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Planet extends Sprite implements IUpdateObj
	{
		public static const MAX_RADIUS:Number = 30.0;
		private static const COL1:Colour = Colour.fromHexi( 0x57e3bd );
		private static const COL2:Colour = Colour.fromHexi( 0x57e38c );
		
		public var radius:Number 					= 0.0;
		public var tradePlanets:Vector.<Planet> 	= null;
		public var port:Port						= null;
		public var tradeShips:Vector.<TradeShip>	= null;
		public var pickUpBuy:PickUp					= null;
		public var pickUpSell:PickUp				= null;
		public var rotSpeed:Number					= 0.0;
		
		public function set player( p:PlayerShip ):void
		{
			p.signalCargo.add( this._onCargoChange );
			this._onCargoChange( p.cargo, p.cargoAmount, p.maxCargoAmount );
		}
		
		public function get active():Boolean { return true; }
		
		public function Planet() 
		{
			this.radius = 10.0 + Math.random() * ( Planet.MAX_RADIUS - 10.0 );			
			this.draw();
			
			// create our port
			this.port = new Port;
			
			// create our pick ups
			this.pickUpBuy 	= new PickUp( this.port.wantGood1, true );
			this.pickUpBuy.x	= this.radius;
			this.addChild( this.pickUpBuy );
			
			this.pickUpSell				= new PickUp( this.port.excessGood1, false );
			this.pickUpSell.x			= -this.radius;
			this.pickUpSell.rotation	= 180.0;
			this.addChild( this.pickUpSell );
			
			this.mouseEnabled 	= false;
			this.mouseChildren	= false;
			
			// set our rotation speed
			this.rotSpeed = -10.0 + Math.random() * 20.0;
			
			DS.addToUpdate( this );
		}
		
		public function update( dt:Number ):void
		{
			this.rotation += this.rotSpeed * dt;
		}
		
		public function createTradeShips():void
		{
			if ( this.tradePlanets == null )
				return;
				
			// create a ship for each planet
			var len:int		= this.tradePlanets.length;
			this.tradeShips = new Vector.<TradeShip>( len, true );
			for ( var i:int = 0; i < len; i++ )
			{
				var s:TradeShip = new TradeShip( this, this.tradePlanets[i] );
				this.parent.addChild( s );
			}
		}
		
		// randomly position our planet in the galaxy
		public function position( bounds:Number ):void
		{
			this.x = -bounds * 0.5 + Math.random() * bounds;
			this.y = -bounds * 0.5 + Math.random() * bounds;
		}
		
		// get the squared dist to another planet
		public function dist( planet:Planet ):Number
		{
			var x:Number = ( planet.x - this.x ) * ( planet.x - this.x );
			var y:Number = ( planet.y - this.y ) * ( planet.y - this.y );
			return x + y;
		}
		
		// add another planet as one to trade with
		public function addTrade( planet:Planet ):void
		{
			if ( this.tradePlanets == null )
				this.tradePlanets = new Vector.<Planet>;
				
			if ( this.tradePlanets.indexOf( planet ) != -1 )
				return;
			this.tradePlanets.push( planet );
		}
		
		// draw our planet
		public function draw():void
		{
			this.graphics.clear();
			this.graphics.lineStyle( 2.0, 0x108f6c );
			this.graphics.beginFill( Colour.fromBlend( Planet.COL1, Planet.COL2, Math.random() ).rgb );
			this.graphics.drawCircle( 0.0, 0.0, this.radius );
			this.graphics.endFill();
			
			var size:Number = this.radius / Planet.MAX_RADIUS;
			var b:Bitmap 	= new Bitmap( DS.getAsset( Resources.PLANET_IMAGE ).bitmapData );
			b.scaleX		= size;
			b.scaleY		= size;
			b.x				= -b.width * 0.5;
			b.y				= -b.height * 0.5;
			this.addChild( b );
			
			// put a glow on there
			this.filters = [new GlowFilter( 0xffffbe, 1.0, 6 * size, 6 * size )];
		}
		
		private function _onCargoChange( cargo:Trade, amount:int, max:int ):void
		{
			// if the player doesn't have cargo, turn off the buy one and highlight the sell one
			if ( cargo == null )
			{
				this.pickUpBuy.enabled 	= false;
				this.pickUpSell.enabled	= true;
			}
			else
			{
				// only turn on buy if it's the right type of trade
				this.pickUpSell.enabled = false;
				this.pickUpBuy.enabled	= ( this.pickUpBuy.trade == cargo );
			}
		}
		
	}

}