package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Mood;
	import com.divillysausages.escvel.constants.Trade;
	import com.divillysausages.escvel.gui.Trail;
	import flash.display.Bitmap;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class PlayerShip extends Sprite implements IUpdateObj
	{
		public var speed:Number 		= 30.0;
		public var maxSpeed:Number		= 1.0;
		public var isTrading:Boolean 	= false;
		public var credits:int			= 0;
		public var cargo:Trade			= null;
		public var cargoPlanet:Planet	= null;
		public var cargoAmount:int		= 0;
		public var maxCargoAmount:int	= 4;
		public var shieldRadius:Number	= 20.0;
		public var shield:Sprite		= null;
		public var vel:Point			= new Point;
		
		public var signalCredits:Signal 	= null;
		public var signalCargo:Signal 		= null;
		public var signalGotHit:Signal		= null;
		public var signalDelivered:Signal	= null;
		
		private var m_dir:Point 					= new Point;
		private var m_galaxy:Galaxy					= null;
		private var m_dirArrows:Vector.<DirArrow>	= new Vector.<DirArrow>;
		private var m_currShootTime:Number			= 0.0;
		private var m_shootTime:Number				= 0.1;
		private var m_trail:Trail					= null;
		private var m_collisions:Vector.<int>		= null;
		
		public function get active():Boolean { return true; }
		
		public function PlayerShip( galaxy:Galaxy ) 
		{
			this.m_galaxy = galaxy;
			this.draw();
			
			this.m_collisions = new Vector.<int>( this.m_galaxy.planets.length, true );
			
			// add shield
			this.shield 		= new Sprite;
			this.shield.alpha	= 0.0;
			var b:Bitmap		= new Bitmap( DS.getAsset( Resources.PLAYER_SHIELD_IMAGE ).bitmapData );
			b.x					= -b.width * 0.5;
			b.y					= -b.height * 0.5;
			this.shield.addChild( b );
			
			// create our signals
			this.signalCredits 		= new Signal( int );
			this.signalCargo		= new Signal( Trade, int, int );
			this.signalGotHit		= new Signal;
			this.signalDelivered	= new Signal;
			
			this.signalCargo.add( this._onCargoChange );
			
			this.mouseEnabled 	= false;
			this.mouseChildren	= false;
			
			// reset
			this.reset();
			
			DS.addToUpdate( this );
		}
		
		public function update( dt:Number ):void
		{				
			// create our trail
			if ( this.m_trail == null )
				this.m_trail = new Trail( this.parent, this.parent.getChildIndex( this ) );
				
			// rotate towards the mouse
			this.m_dir.x 		= this.m_galaxy.mouseX - this.x;
			this.m_dir.y 		= this.m_galaxy.mouseY - this.y;
			var prevRot:Number	= this.rotation;
			if ( this.m_dir.length > 30.0 )
				this.rotation = MathHelper.radiansToDegrees( MathHelper.radianAngle( this.m_dir ) );
			
			// normalise dir
			this.m_dir.normalize( 1.0 );
			
			// if space, move
			if ( Input.isMousePressed && !this.isTrading )
			{
				this.vel.x += this.m_dir.x * this.speed * dt;
				this.vel.y += this.m_dir.y * this.speed * dt;
				
				// clamp
				MathHelper.clamp( this.vel.x, -this.maxSpeed, this.maxSpeed );
				MathHelper.clamp( this.vel.y, -this.maxSpeed, this.maxSpeed );
			}
				
			// move
			this.x += this.vel.x;
			this.y += this.vel.y;
			
			// update our shield
			this.shield.x = this.x;
			this.shield.y = this.y;
			if ( this.shield.parent == null )
				this.parent.addChild( this.shield );
			if ( this.shield.alpha > 0.0 )
				this.shield.alpha -= 0.1;
			
			// move the galaxy so we keep it in focus (move ahead a bit)
			var gx:Number = ( DS.stage.stageWidth * 0.5 * this.m_galaxy.scaleX ) - this.x - this.vel.x * 25.0;
			var gy:Number = ( DS.stage.stageHeight * 0.5 * this.m_galaxy.scaleY ) - this.y - this.vel.y * 25.0;
			gx = MathHelper.damp( this.m_galaxy.x, gx, dt, 0.4, 0.1 );
			gy = MathHelper.damp( this.m_galaxy.y, gy, dt, 0.4, 0.1 );
			this.m_galaxy.x = gx;
			this.m_galaxy.y = gy;
			
			// reduce vel
			this.vel.x *= 0.9;
			this.vel.y *= 0.9;
			
			if ( this.vel.x <= 0.01 && this.vel.x >= -0.01 )
				this.vel.x = 0.0;
			if ( this.vel.y <= 0.01 && this.vel.y >= -0.01 )
				this.vel.y = 0.0;
			
			// update our trail (+5.0 for our back width)
			if( this.vel.x != 0.0 || this.vel.y != 0.0 )
				this.m_trail.update( this.x - this.m_dir.x * 5.0, this.y - this.m_dir.y * 5.0, MathHelper.lerp( prevRot, this.rotation, 0.1 ) );
				
			// position our dir arrows
			for each( var arrow:DirArrow in this.m_dirArrows )
			{
				var dist:Number = 60.0;
				if ( arrow.dist > ( dist + arrow.planet.radius ) )
				{
					arrow.x = this.x + arrow.dir.x * dist;
					arrow.y = this.y + arrow.dir.y * dist;
				}
				else
				{
					arrow.x = this.x + arrow.dir.x * ( arrow.dist - arrow.planet.radius );
					arrow.y = this.y + arrow.dir.y * ( arrow.dist - arrow.planet.radius );
				}
			}
			
			// check if we're close enough to a planet to pick shit up
			var i:int = 0;
			for each( var planet:Planet in this.m_galaxy.planets )
			{					
				var dx:Number 	= this.x - planet.x;
				var dy:Number 	= this.y - planet.y;
				dist 			= Math.sqrt( dx * dx + dy * dy );
				
				// check for collisions
				if ( dist < planet.radius + 5.0 )
					this.m_collisions[i] = 10; // frames
				else
				{
					this.m_collisions[i]--;
					if ( this.m_collisions[i] < 0 )
						this.m_collisions[i] = 0;
				}
				var col:int = this.m_collisions[i];
				i++;
				
				if ( dist > planet.radius + 20.0 || dist < planet.radius ) // outside pickup zone, or inside the planet
					continue;
					
				// if we don't have a cargo, then check for the buy pickup
				if ( this.cargo == null )
				{
					if ( planet.pickUpSell.hitTestObject( this ) )
					{
						this.buy( planet, planet.port.excessGood1, Trade.getBasePrice( planet.port.excessGood1 ) );
						if ( this.cargo != null ) // we bought something
						{
							// check for clean pickup
							if ( col == 0 )
								DS.callLater( this._checkForCleanFlight, 180, i - 1, false );
							break;
						}
					}
				}
				else
				{
					if ( planet.port.wantGood1 != this.cargo )
						continue;
						
					// check for the sell cargo
					if ( planet.pickUpBuy.hitTestObject( this ) )
					{
						// get the base price
						var base:int 			= Trade.getBasePrice( this.cargo );
						var planetDist:Number	= this.cargoPlanet.dist( planet );
						var price:int			= 0;
						var maxDist:int			= 400000; // dist to get max price
						var normalDist:int		= 300000; // dist to get normal price
						var pcRaw:Number		= 0.0;
						var pc:int				= 0;
						if ( planetDist > maxDist )
						{
							DS.soundManager.playSound( Resources.GOOD_DEAL_SOUND, 0, 0.6 );
							pcRaw	= 1.0 + 0.1 * Math.random();
							pc		= int( pcRaw * 100 ) - 100;
							price 	= int( base * pcRaw ); // max price
						}
						else if ( planetDist > normalDist )
						{
							DS.soundManager.playSound( Resources.BUY_SOUND, 0, 0.6 );
							pcRaw	= 0.9 + 0.3 * ( ( planetDist - normalDist ) / ( maxDist - normalDist ) ) + 0.1 * Math.random();
							pc		= int( pcRaw * 100 ) - 100;
							price 	= int( base * pcRaw ); // 100% to max
						}
						else
						{
							DS.soundManager.playSound( Resources.BAD_DEAL_SOUND, 0, 0.6 );
							pcRaw	= 0.7 * ( planetDist / normalDist ) + 0.1 * Math.random();
							pc		= int( pcRaw * 100 ) - 100;
							price 	= int( base * 0.2 + base * pcRaw ); // < 100%
						}
				
						// feedback
						ObjectPools.getFeedback( ( ( pc > 0 ) ? "+" : "" ) + pc + "%",
							( pc > 0 ) ? Mood.GOOD : ( pc == 0 ) ? Mood.NORMAL : Mood.BAD, 
							this.parent.x + this.x,
							this.parent.y + this.y - 20.0,
							true );
							
						this.sell( price );
						
						// check for clean sell
						if ( col == 0 )
							DS.callLater( this._checkForCleanFlight, 180, i - 1, true );
							
						this.signalDelivered.dispatch();
						
						break;
					}
				}
			}
			
			// shoot
			//this.m_currShootTime += dt;
			//if ( this.m_currShootTime >= this.m_shootTime )
				//this.m_currShootTime = this.m_shootTime;
			//if ( Input.isPressed( Key.SPACE ) && this.m_currShootTime >= this.m_shootTime )
			//{
				//var b:Bullet 	= ObjectPools.getBullet();
				//if ( b == null )
					//return;
					//
				//b.x				= this.x;
				//b.y				= this.y;
				//b.setDir( this.m_dir.x, this.m_dir.y );
				//this.parent.addChildAt( b, this.parent.getChildIndex( this ) );
				//
				//this.m_currShootTime -= this.m_shootTime;
			//}
		}
		
		public function reset():void
		{
			// clear any cargo
			this.cargoPlanet	= null;
			this.cargo 			= null;
			this.cargoAmount	= 0;
			this.credits		= 5000;
			this.x				= 0.0;
			this.y				= 0.0;
			
			this.signalCargo.dispatch( this.cargo, this.cargoAmount, this.maxCargoAmount );
			this.signalCredits.dispatch( this.credits );
		}
		
		public function giveCredits( credits:int ):void
		{
			this.credits += credits;
			this.signalCredits.dispatch( this.credits );
				
			// feedback
			DS.callLater( ObjectPools.getFeedback, 200, 
			/*ObjectPools.getFeedback( */"BONUS! +" + credits + " credits",
				Mood.GOOD,
				this.parent.x + this.x,
				this.parent.y + this.y + 20.0,
				true );
		}
		
		public function buy( planet:Planet, trade:Trade, cost:int ):void
		{
			// find out how many we can afford
			var max:int = int( this.credits / cost );
			if ( max > this.maxCargoAmount )
				max = this.maxCargoAmount;
				
			if ( max == 0 )
				return;
				
			DS.soundManager.playSound( Resources.BUY_SOUND );
				
			this.cargo 			= trade;
			this.cargoPlanet	= planet;
			this.cargoAmount	= max;
			this.credits		-= cost * max;
				
			// feedback
			ObjectPools.getFeedback( "+" + this.cargoAmount + Trade.toString( this.cargo ),
				Mood.NORMAL, 
				this.parent.x + this.x,
				this.parent.y + this.y - 5.0,
				true );
			
			this.signalCargo.dispatch( this.cargo, this.cargoAmount, this.maxCargoAmount );
			this.signalCredits.dispatch( this.credits );
		}
		
		public function sell( cost:int ):void
		{
			//if( cost > 0 )
				//DS.soundManager.playSound( Resources.SELL_SOUND, 0, 0.8 );
			
			this.credits 		+= this.cargoAmount * cost;
			this.cargo			= null;
			this.cargoPlanet	= null;
			this.cargoAmount	= 0;
			
			this.signalCargo.dispatch( this.cargo, this.cargoAmount, this.maxCargoAmount );
			this.signalCredits.dispatch( this.credits );
		}
		
		public function collideWithPirate( pirate:Pirate ):void
		{	
			DS.soundManager.playSound( Resources.HIT_SOUND, 0, 0.6 );
			
			// show our shield
			this.shield.rotation 	= MathHelper.radiansToDegrees( Math.atan2( pirate.y - this.y, pirate.x - this.x ) );
			this.shield.alpha 		= 1.0;
			
			if ( this.cargo != null )
			{
				// hold the name
				var cargoName:String = Trade.toString( this.cargo );
				
				// drop some
				this.cargoAmount--;
				if ( this.cargoAmount <= 0 )
				{
					this.cargo 			= null;
					this.cargoPlanet	= null;
					this.cargoAmount 	= 0;
				}
				
				// feedback
				ObjectPools.getFeedback( "-1 " + cargoName, 
					Mood.BAD, 
					this.parent.x + this.x, 
					this.parent.y + this.y,
					true );
				
				// dispatch our signal
				this.signalCargo.dispatch( this.cargo, this.cargoAmount, this.maxCargoAmount );
			}
			else if ( this.credits > 0 )
			{
				var prev:int	= this.credits;
				this.credits 	-= 200.0 + int( Math.random() * 200.0 );
				if ( this.credits < 0 )
					this.credits = 0;
				
				// feedback
				ObjectPools.getFeedback( "-" + ( prev - this.credits ) + " credits", 
					Mood.BAD, 
					this.parent.x + this.x, 
					this.parent.y + this.y,
					true );
					
				this.signalCredits.dispatch( this.credits );
			}
			
			this.signalGotHit.dispatch();
		}
		
		public function addVel( vx:Number, vy:Number ):void
		{
			this.vel.x += vx;
			this.vel.y += vy;
			
			// clamp it
			MathHelper.clamp( this.vel.x, -this.maxSpeed, this.maxSpeed );
			MathHelper.clamp( this.vel.y, -this.maxSpeed, this.maxSpeed );
		}
		
		public function draw():void
		{
			var b:Bitmap 	= new Bitmap( DS.getAsset( Resources.PLAYER_IMAGE ).bitmapData );
			b.y				= -b.height * 0.5;
			b.x				= -5.0;
			this.addChild( b );
			//this.graphics.clear();
			//this.graphics.lineStyle( 1.0, 0x2b8e14 );
			//this.graphics.beginFill( 0x58d83a );
			//
			//var h:Number = 5;
			//
			//this.graphics.moveTo( -h, -h );
			//this.graphics.lineTo( h * 2.0, 0.0 );
			//this.graphics.lineTo( -h, h );
			//this.graphics.lineTo( -h, -h );
			//
			//this.graphics.endFill();
		}
		
		private function _onCargoChange( trade:Trade, amount:int, max:int ):void
		{
			// if no trade, clear all arrows
			if ( trade == null )
			{
				for each( var arrow:DirArrow in this.m_dirArrows )
					ObjectPools.returnDirArrow( arrow );
				this.m_dirArrows.length = 0;
			}
			else
			{
				// find all the planets that need it and point them out
				for each( var planet:Planet in this.m_galaxy.planets )
				{
					if ( planet.port.wantGood1 == trade || planet.port.wantGood2 == trade )
					{
						arrow 			= ObjectPools.getDirArrow( planet );
						arrow.player	= this;
						this.m_dirArrows.push( arrow );
					}
				}
			}
		}
		
		private function _checkForCleanFlight( planet:int, wasDropOff:Boolean ):void
		{
			if ( this.m_collisions[planet] > 0 )
				return;
				
			DS.soundManager.playSound( Resources.PICKUP_PERFECT_SOUND, 0, 0.5 );
				
			var creds:int = 0;
			if ( wasDropOff )
			{
				creds = int( vel.length * 100 ); // 500;
				ObjectPools.getFeedback( "Clean dropoff! +" + creds + " credits",
					Mood.GOOD, 
					this.parent.x + this.x, 
					this.parent.y + this.y,
					true );
				this.credits += creds;
			}
			else
			{
				creds = int( vel.length * 50 );// 100;
				ObjectPools.getFeedback( "Clean pickup! +" + creds + " credits", 
					Mood.GOOD, 
					this.parent.x + this.x, 
					this.parent.y + this.y,
					true );
				this.credits += creds;
			}
		}
		
	}

}