package com.divillysausages.escvel.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.escvel.constants.Trade;
	import flash.display.Shape;
	import flash.geom.Point;
	
	/**
	 * trader ship that goes between planets
	 * @author Damian Connolly
	 */
	public class TradeShip extends Shape implements IUpdateObj
	{
		public var homePlanet:Planet	= null;
		public var destPlanet:Planet	= null;
		public var trade:Trade			= null;
		public var amount:int			= 0;
		public var speed:Number			= 15;
		public var docking:Planet		= null;
		public var currDest:Planet		= null;
		
		private var m_currRestTime:Number 	= 0.0;
		private var m_restTime:Number		= 5.0 + Math.random() * 5.0;
		private var m_dir:Point				= new Point;
		
		public function get active():Boolean { return true; }
		
		public function TradeShip( home:Planet, dest:Planet ) 
		{
			this.homePlanet = home;
			this.destPlanet = dest;
			this.docking	= this.homePlanet; // docking at the home planet for the minute
			
			// start off at our home planet
			this.x = this.homePlanet.x;
			this.y = this.homePlanet.y;
			
			// we're on the planet, so not visible
			this.visible = false;
			
			// draw our graphics
			this.draw();
			
			// add to update
			DS.addToUpdate( this );
		}
		
		public function draw():void
		{
			this.graphics.clear();
			this.graphics.lineStyle( 1.0, 0x6a877f );
			this.graphics.beginFill( 0xc2d4cf );
			
			var h:Number = 2.5;
			
			this.graphics.moveTo( -h, -h );
			this.graphics.lineTo( h * 2.0, 0.0 );
			this.graphics.lineTo( -h, h );
			this.graphics.lineTo( -h, -h );
			
			this.graphics.endFill();
		}
		
		public function update( dt:Number ):void
		{
			// we're docking
			if ( this.docking != null )
			{
				this.m_currRestTime += dt;
				if ( this.m_currRestTime < this.m_restTime )
					return;
			}
			
			// get some goods and start up
			if ( this.trade == null )
			{
				this.trade 	= this.docking.port.excessGood1;// ( Math.random() < 0.5 ) ? this.docking.port.excessGood1 : this.docking.port.excessGood2;
				this.amount	= 1 + int( Math.random() * 3.0 );
			}
			
			// take our dest
			if ( this.currDest == null )
			{
				this.currDest 	= ( this.docking == this.homePlanet ) ? this.destPlanet : this.homePlanet;
				this.docking	= null;
			}
			
			// move
			this.m_dir.x 	= this.currDest.x - this.x;
			this.m_dir.y 	= this.currDest.y - this.y;
			var len:Number	= this.m_dir.length;
			this.m_dir.normalize( 1.0 );
			
			// we're visible
			this.visible = true;
			
			this.x 			+= this.m_dir.x * this.speed * dt;
			this.y 			+= this.m_dir.y * this.speed * dt;
			this.rotation 	= MathHelper.radiansToDegrees( MathHelper.radianAngle( this.m_dir ) );
			
			// if we're close enough, dock
			if ( len < this.currDest.radius )
			{
				this.docking 		= this.currDest;
				this.currDest		= null;
				this.trade			= null;
				this.m_currRestTime	= 0.0;
				this.visible		= false; // hide as we're docking
			}
		}
		
	}

}