package com.divillysausages.escvel.state
{
	import com.divillysausages.ds.core.GameStateManager;
	import com.divillysausages.ds.DS;
	import flash.errors.IllegalOperationError;
	
	/**
	 * The controlling game state
	 * @author Damian Connolly
	 */
	public class Game extends GameStateManager 
	{
	
		/**
		 * The init state
		 */
		public static const STATE_INIT:String = "init";
		
		/**
		 * The image check state
		 */
		public static const STATE_CHECK:String = "check";
		
		/**
		 * The hacked state
		 */
		public static const STATE_HACKED:String = "hacked";
		
		/**
		 * The DS logo state
		 */
		public static const STATE_DS:String = "ds";
		
		/**
		 * The main menu state
		 */
		public static const STATE_MAIN_MENU:String = "main menu";
		
		/**
		 * The play state
		 */
		public static const STATE_PLAY:String = "play";
		
		/*******************************************************************************************/
		
		private static var m_creating:Boolean	= false;// are we currently creating the game?
		private static var m_instance:Game 		= null;	// the game instance
		
		/*******************************************************************************************/
		
		/**
		 * Gets the singleton instance of the Game class
		 */
		public static function get instance():Game 
		{
			if ( Game.m_instance == null ) 
			{
				Game.m_creating	= true;
				Game.m_instance = new Game;
				Game.m_creating = false;
			}
			return Game.m_instance;
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates the game class
		 */
		public function Game() 
		{
			// make sure we're going through the singleton
			if ( !Game.m_creating )
				throw new IllegalOperationError( "The Game class is a singleton, use the instance property" );
				
			// get the main class
			var m:Main = DS.main as Main;
			
			// add our states
			this.addState( new StateInit( m, Game.STATE_INIT ) );
			this.addState( new StateCheck( m, Game.STATE_CHECK ) );
			this.addState( new StateHacked( m, Game.STATE_HACKED ) );
			this.addState( new StateDS( m, Game.STATE_DS ) );
			this.addState( new StateMainMenu( m, Game.STATE_MAIN_MENU ) );
			this.addState( new StatePlay( m, Game.STATE_PLAY ) );
		}
		
	}

}