package com.divillysausages.escvel.state
{
	import com.divillysausages.ds.core.GameState;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.ImageSummer;
	import com.divillysausages.ds.security.ImageVerifier;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.assets.Security;
	import com.divillysausages.escvel.constants.Style;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * Checks to see if all our images are ok
	 * @author Damian Connolly
	 */
	public class StateCheck extends GameState 
	{
		
		/*******************************************************************************************/
		
		private var m_text:TextField	= null; // our textfield
		private var m_main:Main			= null;	// our main image
		private var m_checking:Boolean	= false;// are we checking yet?
		private var m_wait:int			= 0;	// how long to wait
		
		CONFIG::debug
		{
			// we show the summer in debug mode
			private var m_summer:ImageSummer = null;
		}
		CONFIG::release
		{
			// we verify in release mode
			private var m_verifier:ImageVerifier = null;
		}
		
		/*******************************************************************************************/
		
		/**
		 * Creates our state check
		 * @param m The Main class
		 * @param name The name of this state
		 */
		public function StateCheck( m:Main, name:String ) 
		{
			this.m_main = m;
			super( name );
			
			// create our text
			this._createText();
		}
		
		/**
		 * Start the state
		 */
		override public function begin():void
		{
			// add our text
			DS.stage.addChild( this.m_text );
			
			CONFIG::debug
			{
				// create our summer
				this.m_summer = new ImageSummer( "hexcombat" );
			}
			CONFIG::release
			{
				// create our verifier
				this.m_verifier = new ImageVerifier( this._onCheckOK, this._onCheckFail );
			}
			
			this.m_checking = false;
			this.m_wait		= 0;
		}
		
		/**
		 * End the state
		 */
		override public function end():void
		{
			// remove our text
			DS.stage.removeChild( this.m_text );
			
			CONFIG::debug
			{
				// destroy our summer
				this.m_summer.destroy();
				this.m_summer = null;
			}
			CONFIG::release
			{
				// destroy our verifier
				this.m_verifier.destroy();
				this.m_verifier = null;
			}
		}
		
		/**
		 * Updates the StateCheck state
		 * @param dt	The delta time
		 */
		override public function update(dt:Number):void 
		{
			if ( this.m_wait < 3 )
			{
				this.m_wait++;
				return;
			}
			
			if ( !this.m_checking )
			{
				CONFIG::debug
				{
					// TODO: set the images to sum
					this.m_summer.images = Security.IMAGES;
				}
				CONFIG::release
				{
					// TODO: set the images to verify and their sums
					this.m_verifier.images	= Security.IMAGES;
					this.m_verifier.sums	= Security.SUMS;
				}
				
				// we're checking now, so it's ok
				this.m_checking = true;
			}
		}
		
		/*******************************************************************************************/
		
		// create the textfield that we'll use
		private function _createText():void
		{
			var tf:TextFormat 				= new TextFormat( Resources.GAME_FONT, Style.FONT_SIZE, Style.TEXT_COLOUR );
			this.m_text 					= new TextField;
			this.m_text.defaultTextFormat 	= tf;
			this.m_text.autoSize			= TextFieldAutoSize.LEFT;
			this.m_text.text				= "LOADING...";
			this.m_text.selectable			= false;
			this.m_text.mouseEnabled		= false;
			this.m_text.mouseWheelEnabled	= false;
			this.m_text.embedFonts			= true;
			this.m_text.x					= DS.stage.stageWidth * 0.5 - this.m_text.width * 0.5;
			this.m_text.y					= DS.stage.stageHeight * 0.5 - this.m_text.height * 0.5;
		}
		
		// called when everything is ok
		private function _onCheckOK():void
		{
			Game.instance.changeState( Game.STATE_DS );
		}
		
		// called when there was a problem
		private function _onCheckFail():void
		{
			Game.instance.changeState( Game.STATE_HACKED );
		}
		
	}

}