package com.divillysausages.escvel.state
{
	import com.divillysausages.ds.core.GameState;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.security.SafeNumber;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Style;
	import flash.display.Bitmap;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * The state where we show up our logo
	 * @author Damian Connolly
	 */
	public class StateDS extends GameState 
	{
		
		/*******************************************************************************************/
		
		private var m_main:Main				= null;	// a reference to the main class
		private var m_logo:Bitmap 			= null;	// our logo
		private var m_time:SafeNumber		= null;	// the time to hold on our logo
		private var m_currTime:SafeNumber	= null;	// our current time
		private var m_text:TextField		= null;	// our text
		private var m_playedSound:Boolean	= false;// have we played the sound yet?
		
		/*******************************************************************************************/
		
		/**
		 * Creates our DS state
		 * @param m		A reference to the main class
		 * @param name	Our name
		 */
		public function StateDS( m:Main, name:String ) 
		{
			this.m_main = m;
			super( name );
			
			// our time objects
			this.m_time 	= new SafeNumber( 3.0 );
			this.m_currTime	= new SafeNumber( 0.0 );
		}
		
		/**
		 * Starts the DS state
		 */
		override public function begin():void 
		{	
			// create our logo and text
			this._createLogo();
			this._createText();
			
			// show our logo
			this.m_logo.visible 	= true;
			this.m_logo.alpha		= 0.0;
			this.m_text.visible		= true;
			this.m_text.alpha		= 0.0;
			this.m_currTime.value	= 0.0;
			
			this.m_playedSound = false;
			
			// add our objects
			DS.canvas.addChild( this.m_logo );
			DS.canvas.addChild( this.m_text );
		}
		
		/**
		 * When our state has ended
		 */
		override public function end():void 
		{
			// remove our objects
			DS.canvas.removeChild( this.m_logo );
			DS.canvas.removeChild( this.m_text );
			
			// reset
			this.m_logo.visible = false;
			this.m_text.visible	= false;
		}
		
		/**
		 * Shows our logo for a period of time
		 * @param dt	The time since the last call to update
		 */
		override public function update(dt:Number):void 
		{
			// fade in our logo
			if ( this.m_currTime.value < this.m_time.value )
			{
				if ( this.m_logo.alpha < 1.0 )
				{
					this.m_logo.alpha += 0.05;
					this.m_text.alpha += 0.05;
				}
				else
				{
					this.m_logo.alpha = 1.0;
					this.m_text.alpha = 1.0;
					
					if ( !this.m_playedSound )
					{
						DS.soundManager.playSound( Resources.DS_SOUND );
						this.m_playedSound = true;
					}
				}
					
				// add our time
				this.m_currTime.value += dt;
			}
			else
			{
				if ( this.m_logo.alpha > 0.0 )
				{
					this.m_logo.alpha -= 0.05;
					this.m_text.alpha -= 0.05;
				}
				else
					Game.instance.changeState( Game.STATE_MAIN_MENU );
			}
		}
		
		/*******************************************************************************************/
		
		// create the logo
		private function _createLogo():void
		{
			if ( this.m_logo != null )
				return;
				
			// create our logo
			this.m_logo 			= new Bitmap;
			this.m_logo.bitmapData	= DS.getAsset( Resources.DS_IMAGE ).bitmapData;
			this.m_logo.x			= DS.stage.stageWidth * 0.5 - this.m_logo.width * 0.5;
			this.m_logo.y			= DS.stage.stageHeight * 0.5 - this.m_logo.height * 0.5;
		}
		
		// create the textfield that we'll use
		private function _createText():void
		{
			if ( this.m_text != null )
				return;
				
			var tf:TextFormat 				= new TextFormat( Resources.GAME_FONT, Style.FONT_SIZE, Style.LINK_COLOUR );
			this.m_text 					= new TextField;
			this.m_text.defaultTextFormat 	= tf;
			this.m_text.autoSize			= TextFieldAutoSize.LEFT;
			this.m_text.htmlText			= "<a href='http://divillysausages.com' target='_blank'>http://divillysausages.com</a>";
			this.m_text.selectable			= false;
			this.m_text.mouseEnabled		= true;
			this.m_text.mouseWheelEnabled	= false;
			this.m_text.embedFonts			= true;
			this.m_text.x					= DS.stage.stageWidth * 0.5 - this.m_text.width * 0.5;
			this.m_text.y					= this.m_logo.y + this.m_logo.height + 20.0;
		}
		
	}

}