package com.divillysausages.escvel.state
{
	import com.divillysausages.ds.core.GameState;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.external.ExternalAPI;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.constants.Style;
	import flash.display.Shape;
	import flash.external.ExternalInterface;
	import flash.filters.BlurFilter;
	import flash.text.StyleSheet;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	/**
	 * The state we go to if we detect that the game's been hacked
	 * @author Damian Connolly
	 */
	public class StateHacked extends GameState 
	{
		
		/*******************************************************************************************/
		
		private var m_main:Main			= null; // our main class
		private var m_graphics:Shape	= null; // the graphics that we'll use to cover the stage (no clicking)
		private var m_text:TextField 	= null; // the text to show
		
		/*******************************************************************************************/
		
		/**
		 * Create our hacked screen
		 * @param m A reference to the Main class
		 * @param name The name of this class
		 */
		public function StateHacked( m:Main, name:String ) 
		{
			this.m_main = m;
			super( name );
			
			// draw our bg
			this.m_graphics = new Shape;
			this.m_graphics.graphics.beginFill( Style.BG_COLOUR.rgb );
			this.m_graphics.graphics.drawRect( 0.0, 0.0, DS.stage.stageWidth, DS.stage.stageHeight );
			this.m_graphics.graphics.endFill();
			
			// create our text
			this._createText();
		}
		
		/**
		 * Show the hacked screen
		 */
		override public function begin():void
		{
			// blur the bg
			DS.canvas.filters = [new BlurFilter];
			
			// add us
			DS.stage.addChild( this.m_graphics );
			DS.stage.addChild( this.m_text );
			if ( ExternalInterface.available )
				ExternalAPI.trackEvent( "[Hacked screen] " + ExternalInterface.call( "window.location.href" ) );
			else
				ExternalAPI.trackEvent( "[Hacked screen] " + DS.stage.loaderInfo.loaderURL );
		}
		
		/**
		 * We can't hide the hacked screen
		 */
		override public function end():void
		{
			// don't hide
			Game.instance.changeState( Game.STATE_HACKED );
		}
		
		/*******************************************************************************************/
		
		// create the textfield
		private function _createText():void
		{
			var tf:TextFormat 	= new TextFormat( Resources.GAME_FONT, Style.FONT_SIZE, Style.TEXT_COLOUR );
			tf.align			= TextFormatAlign.CENTER;
			
			var msg:String = "<font size='" + Style.FONT_SIZE_LARGE + "'>Sorry</font>\n<font size='" + 
								Style.FONT_SIZE + "'>It looks like this game has been hacked\n\n" + 
								"Please support the original devs. You can find more on the original game at:" + 
								"\n<a href=\"http://divillysausages.com/tiny_traders\">http://divillysausages.com/tiny_traders</a>" +
								"\n\nThank you</font>";
								
			// our stylesheet for the link
			var style:StyleSheet 	= new StyleSheet();
			var link:Object 		= new Object;
			link.color 				= Style.LINK_COLOUR_HTML;
			style.setStyle( "a", link );
			
			// our textfield
			this.m_text 					= new TextField;
			this.m_text.defaultTextFormat	= tf;
			this.m_text.styleSheet			= style;
			this.m_text.htmlText			= msg.toUpperCase();
			this.m_text.selectable			= false;
			this.m_text.embedFonts			= true;
			this.m_text.multiline			= true;
			this.m_text.width				= 400.0;
			this.m_text.wordWrap			= true;
			this.m_text.height				= this.m_text.textHeight + 4.0;
			this.m_text.x 					= DS.stage.stageWidth * 0.5 - this.m_text.width * 0.5;
			this.m_text.y					= DS.stage.stageHeight * 0.5 - this.m_text.height * 0.5;
		}
		
	}

}