package com.divillysausages.escvel.state 
{
	import com.divillysausages.ds.core.GameState;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.fx.ScreenShaker;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import com.divillysausages.escvel.assets.Resources;
	import com.divillysausages.escvel.gui.BuyDialog;
	import com.divillysausages.escvel.gui.Instructions;
	import com.divillysausages.escvel.gui.ParticleRenderer;
	import com.divillysausages.escvel.gui.PickUpEmitter;
	import com.divillysausages.escvel.gui.PlayerHUD;
	import com.divillysausages.escvel.gui.SpeedBonus;
	import com.divillysausages.escvel.gui.Stars;
	import com.divillysausages.escvel.logic.Galaxy;
	import com.divillysausages.escvel.logic.MiniMap;
	import com.divillysausages.escvel.logic.Pirate;
	import com.divillysausages.escvel.logic.Planet;
	import com.divillysausages.escvel.logic.PlayerShip;
	import flash.display.Bitmap;
	import flash.events.KeyboardEvent;
	import flash.filters.GlowFilter;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class StatePlay extends GameState
	{
		private var m_main:Main 						= null;
		private var m_stars:Stars						= null;
		private var m_galaxy:Galaxy						= null;
		private var m_player:PlayerShip					= null;
		private var m_miniMap:MiniMap					= null;
		private var m_pirates:Vector.<Pirate>			= null;
		private var m_buyDialog:BuyDialog				= null;
		private var m_playerHUD:PlayerHUD				= null;
		private var m_shaker:ScreenShaker				= null;
		private var m_instructions:Instructions			= null;
		private var m_particleRenderer:ParticleRenderer	= null;
		private var m_pickUpParticles:PickUpEmitter		= null;
		private var m_speedBonus:SpeedBonus				= null;
		
		public function StatePlay( m:Main, name:String ) 
		{
			this.m_main = m;
			super( name );
		}
		
		override public function begin():void 
		{			
			// stars
			this.m_stars = new Stars;
			DS.canvas.addChild( this.m_stars );
			
			// the galaxy
			this.m_galaxy 	= new Galaxy;
			this.m_galaxy.x = DS.stage.stageWidth * 0.5;
			this.m_galaxy.y = DS.stage.stageHeight * 0.5;
			DS.canvas.addChild( this.m_galaxy );
			
			// minimap
			this.m_miniMap 		= new MiniMap( this.m_galaxy );
			this.m_miniMap.x	= DS.stage.stageWidth - this.m_miniMap.width;
			this.m_miniMap.y	= DS.stage.stageHeight - this.m_miniMap.height;
			DS.stage.addChild( this.m_miniMap );
			
			// the player
			this.m_player = new PlayerShip( this.m_galaxy );
			this.m_player.signalGotHit.add( this._onHit );
			this.m_player.signalDelivered.add( this._onDelivered );
			this.m_galaxy.addChild( this.m_player );
			
			// pirates
			var len:int 	= 20;
			this.m_pirates 	= new Vector.<Pirate>( len, true );
			for ( var i:int = 0; i < len; i++ )
			{
				var p:Pirate = new Pirate( this.m_galaxy, this.m_player );
				if ( Math.random() < 0.5 )
				{
					// horizontal
					p.x	= -this.m_galaxy.bounds * 0.5 + Math.random() * this.m_galaxy.bounds;
					p.y	= Math.random() * 100.0 + ( ( Math.random() < 0.5 ) ? -this.m_galaxy.bounds * 0.5 : this.m_galaxy.bounds * 0.5 - 100.0 );
				}
				else
				{
					// vertical
					p.x	= Math.random() * 100.0 + ( ( Math.random() < 0.5 ) ? -this.m_galaxy.bounds * 0.5 : this.m_galaxy.bounds * 0.5 - 100.0 );
					p.y	= -this.m_galaxy.bounds * 0.5 + Math.random() * this.m_galaxy.bounds;
				}
				this.m_galaxy.addChild( p );
				this.m_pirates[i] = p;
			}
			
			// tell all the planets about the player
			for each( var planet:Planet in this.m_galaxy.planets )
				planet.player = this.m_player;
			
			// player hud
			this.m_playerHUD 		= new PlayerHUD;
			this.m_playerHUD.x		= 10.0;
			this.m_playerHUD.y		= DS.stage.stageHeight - this.m_playerHUD.height - 10.0;
			this.m_playerHUD.player	= this.m_player;
			DS.stage.addChild( this.m_playerHUD );
			
			// the screen shaker
			this.m_shaker = new ScreenShaker;
			
			// create the mouse instructions
			this.m_instructions = new Instructions;
			
			// particle renderer
			this.m_particleRenderer = new ParticleRenderer;
			this.m_pickUpParticles	= new PickUpEmitter( this.m_particleRenderer );
			this.m_galaxy.addChild( this.m_particleRenderer );
			
			// speed bonus
			this.m_speedBonus 			= new SpeedBonus;
			this.m_speedBonus.player	= this.m_player;
			
			// buy dialog
			//this.m_buyDialog 	= new BuyDialog;
			//this.m_buyDialog.x	= 10.0;
			//this.m_buyDialog.y	= -this.m_buyDialog.height;
			//DS.stage.addChild( this.m_buyDialog );
			
			// drop cargo
			Input.bind( Key.D, this._onDropCargo );
			Input.bind( Key.R, this._reset );
			Input.bind( Key.S, this._onToggleSound );
			Input.bind( Key.M, this._onToggleMusic );
			
			// mouse listener
			//DS.stage.addEventListener( MouseEvent.MOUSE_DOWN, this._onDown );
			this.m_instructions.start();
			DS.soundManager.playSoundtrack( Resources.MUSIC, 0.8 );
		}
		
		override public function update(dt:Number):void 
		{
			this.m_miniMap.update( this.m_player );
			for each( var p:Pirate in this.m_pirates )
				this.m_miniMap.updatePirate( p );
				
			// update our instructions
			this.m_instructions.update( dt );
			
			// update our stars
			this.m_stars.update( this.m_player.vel );
		}
		
		/*private function _onDown( e:MouseEvent ):void
		{
			var p:Planet = this.m_galaxy.getPlanetAtPos( e.localX, e.localY );
			if ( p != null )
			{
				// get the distance to the planet
				var dx:Number = this.m_player.x - p.x;
				var dy:Number = this.m_player.y - p.y;
				var dist:Number = Math.sqrt( dx * dx + dy * dy );
				if ( dist < p.radius + 60.0 )
				{
					this.m_player.isTrading =  true;
					
					// show the buy dialog
					this.m_buyDialog.planet = p;
					this.m_buyDialog.player = this.m_player;
					TweenLite.to( this.m_buyDialog, 1.0, { y:DS.stage.stageHeight * 0.5 - this.m_buyDialog.height * 0.5, ease: Elastic.easeOut } );
					return;
				}
			}
			
			// the player's not trading
			if( this.m_buyDialog.y <= -this.m_buyDialog.height )
				this.m_player.isTrading = false;
		}*/
		
		private function _onDropCargo():void
		{
			if ( this.m_player.cargo != null )
				this.m_player.sell( 0 );
		}
		
		// reset the game
		private function _reset():void
		{
			this.m_player.reset();
		}
		
		private function _onToggleSound():void
		{
			DS.soundManager.soundsMuted = !DS.soundManager.soundsMuted;
		}
		
		private function _onToggleMusic():void
		{
			DS.soundManager.musicMuted = !DS.soundManager.musicMuted;
		}
		
		// player got hit by a pirate
		private function _onHit():void
		{
			this.m_shaker.shake( 0.2, 10.0 );
		}
		
		// player delivered some cargo
		private function _onDelivered():void
		{
			// move the renderer to the right position
			this.m_particleRenderer.x = this.m_player.x - this.m_particleRenderer.width * 0.5;
			this.m_particleRenderer.y = this.m_player.y - this.m_particleRenderer.height * 0.5;
			this.m_pickUpParticles.emit( this.m_particleRenderer.width * 0.5, this.m_particleRenderer.height * 0.5 );
		}
		
	}

}